@@ -98,14 +98,14 @@ class Collection(object):
         return method(obj, name=name)
 
     @classmethod
-    def from_module(self, module):
+    def from_module(self, module, name=None, config=None):
         """
         Return a new `.Collection` created from ``module``.
 
         Inspects ``module`` for any `.Task` instances and adds them to a new
         `.Collection`, returning it. If any explicit namespace collections
-        exist (named ``ns`` or ``namespace``) they are preferentially loaded
-        instead.
+        exist (named ``ns`` or ``namespace``) a copy of that collection object
+        is preferentially loaded instead.
 
         When the implicit/default collection is generated, it will be named
         after the module's ``__name__`` attribute, or its last dotted section
@@ -114,6 +114,19 @@ class Collection(object):
 
         Explicitly given collections will only be given that module-derived
         name if they don't already have a valid ``.name`` attribute.
+
+        :param name:
+            A string, which if given will override any automatically derived
+            collection name (or name set on the module's root namespace, if it
+            has one.)
+
+        :param config:
+            A dict, used to set config options on the newly created
+            `.Collection` before returning it (saving you a call to
+            `.configure`.)
+            
+            If the imported module had a root namespace object, ``config`` is
+            merged on top of it (i.e. overriding any conflicts.)
         """
         module_name = module.__name__.split('.')[-1]
         # See if the module provides a default NS to use in lieu of creating
@@ -121,29 +134,43 @@ class Collection(object):
         for candidate in ('ns', 'namespace'):
             obj = getattr(module, candidate, None)
             if obj and isinstance(obj, Collection):
-                if not obj.name:
-                    obj.name = module_name
-                return obj
+                # Explicitly given name wins over root ns name which wins over
+                # actual module name.
+                ret = Collection(name or obj.name or module_name)
+                ret.tasks = copy.deepcopy(obj.tasks)
+                ret.collections = copy.deepcopy(obj.collections)
+                ret.default = copy.deepcopy(obj.default)
+                # Explicitly given config wins over root ns config
+                obj_config = copy.deepcopy(obj._configuration)
+                if config:
+                    obj_config.update(config)
+                ret._configuration = obj_config
+                return ret
         # Failing that, make our own collection from the module's tasks.
         tasks = filter(
             lambda x: isinstance(x, Task),
             vars(module).values()
         )
-        collection = Collection(module_name)
+        # Again, explicit name wins over implicit one from module path
+        collection = Collection(name or module_name)
         for task in tasks:
             collection.add_task(task)
+        if config:
+            collection.configure(config)
         return collection
 
-    def add_task(self, task, name=None):
+    def add_task(self, task, name=None, default=None):
         """
-        Adds ``Task`` ``task`` to this collection.
+        Add `.Task` ``task`` to this collection.
 
-        The name the task is bound with is taken, in order, from:
+        :param task: The `.Task` object to add to this collection.
 
-        * The ``name`` kwarg;
-        * If that is not given, the task object's ``name`` attribute;
-        * If that is empty, the task object's wrapped callable's ``.func_name``
-          attribute.
+        :param name:
+            Optional string name to bind to (overrides the task's own
+            self-defined ``name`` attribute and/or any Python identifier (i.e.
+            ``.func_name``.)
+
+        :param default: Whether this task should be the collection default.
         """
         if name is None:
             if task.name:
@@ -159,7 +186,7 @@ class Collection(object):
         self.tasks[name] = task
         for alias in task.aliases:
             self.tasks.alias(alias, to=name)
-        if task.is_default:
+        if default is True or (default is None and task.is_default):
             if self.default:
                 msg = "'%s' cannot be the default because '%s' already is!"
                 raise ValueError(msg % (name, self.default))
@@ -179,6 +206,12 @@ class Collection(object):
         # Insert
         self.collections[name] = coll
 
+    def split_path(self, path):
+        parts = path.split('.')
+        coll = parts.pop(0)
+        rest = '.'.join(parts)
+        return coll, rest
+
     def __getitem__(self, name=None):
         """
         Returns task named ``name``. Honors aliases and subcollections.
@@ -199,9 +232,7 @@ class Collection(object):
                 raise ValueError("This collection has no default task.")
         # Non-default tasks within subcollections
         if '.' in name:
-            parts = name.split('.')
-            coll = parts.pop(0)
-            rest = '.'.join(parts)
+            coll, rest = self.split_path(name)
             return self.collections[coll][rest]
         # Default task for subcollections (via empty-name lookup)
         if name in self.collections:
@@ -259,25 +290,26 @@ class Collection(object):
                 ret[self.subtask_name(coll_name, task_name)] = aliases
         return ret
 
-    @property
-    def configuration(self):
+    def configuration(self, path=None):
         """
-        Return this collection's configuration options as a dict.
-
-        Child/inner collections' configurations are merged on top of this
-        collection's, though the outer collection's value will win in
-        conflicts.
-
-        Multiple child collections' configurations are merged in alphabetical
-        order by attached name (thus childs with 'later' names will win.)
+        Obtain merged configuration values from collection & children.
 
         .. note::
             Merging uses ``copy.deepcopy`` to prevent state bleed.
+
+        :param path:
+            (Optional) Dotted string path, same as used for looking up actual
+            tasks, except without the final task name component. Used to
+            determine which subcollection (if any) has its configuration merged
+            in.
+
+        :returns: A dictionary containing configuration values.
         """
-        # Merge sideways across subcollections first
         ret = {}
-        for subcol in sorted(self.collections.keys()):
-            ret.update(copy.deepcopy(self.collections[subcol].configuration))
+        # Merge subcollections if necessary
+        if path:
+            subcoll, rest = self.split_path(path)
+            ret.update(copy.deepcopy(self.collections[subcoll].configuration(rest)))
         # Then merge in ours so we win conflicts
         ret.update(copy.deepcopy(self._configuration))
         return ret
