@@ -1,7 +1,7 @@
 import operator
 import sys
 
-from spec import Spec, skip, eq_, raises
+from spec import Spec, skip, eq_, raises, assert_raises
 
 from invoke.collection import Collection
 from invoke.tasks import task, Task
@@ -75,6 +75,31 @@ class Collection_(Spec):
         def setup(self):
             self.c = Collection.from_module(load('integration'))
 
+        class parameters:
+            def setup(self):
+                self.mod = load('integration')
+                self.fm = Collection.from_module
+
+            def name_override(self):
+                eq_(self.fm(self.mod).name, 'integration')
+                eq_(
+                    self.fm(self.mod, name='not-integration').name,
+                    'not-integration'
+                )
+
+            def inline_configuration(self):
+                eq_(self.fm(self.mod).configuration(), {})
+                eq_(
+                    self.fm(self.mod, config={'foo': 'bar'}).configuration(),
+                    {'foo': 'bar'}
+                )
+
+            def name_and_config_simultaneously(self):
+                # Test w/ posargs to enforce ordering, just for safety.
+                c = self.fm(self.mod, 'the name', {'the': 'config'})
+                eq_(c.name, 'the name')
+                eq_(c.configuration(), {'the': 'config'})
+
         def adds_tasks(self):
             assert 'print_foo' in self.c
 
@@ -100,6 +125,48 @@ class Collection_(Spec):
             coll = Collection.from_module(load('subcollection_task_name'))
             assert 'explicit_name' in coll.tasks # not 'implicit_name'
 
+        def returns_unique_Collection_objects_for_same_input_module(self):
+            # Ignoring self.c for now, just in case it changes later.
+            # First, a module with no root NS
+            mod = load('integration')
+            c1 = Collection.from_module(mod)
+            c2 = Collection.from_module(mod)
+            assert c1 is not c2
+            # Now one *with* a root NS (which was previously buggy)
+            mod2 = load('explicit_root')
+            c3 = Collection.from_module(mod2)
+            c4 = Collection.from_module(mod2)
+            assert c3 is not c4
+
+        class explicit_root_ns:
+            def setup(self):
+                mod = load('explicit_root')
+                mod.ns.configure({'key': 'builtin', 'otherkey': 'yup'})
+                mod.ns.name = 'builtin_name'
+                self.unchanged = Collection.from_module(mod)
+                self.changed = Collection.from_module(
+                    mod,
+                    name='override_name',
+                    config={'key': 'override'}
+                )
+
+            def inline_config_with_root_namespaces_overrides_builtin(self):
+                eq_(self.unchanged.configuration()['key'], 'builtin')
+                eq_(self.changed.configuration()['key'], 'override')
+
+            def inline_config_overrides_via_merge_not_replacement(self):
+                assert 'otherkey' in self.changed.configuration()
+
+            def inline_name_overrides_root_namespace_object_name(self):
+                eq_(self.unchanged.name, 'builtin_name')
+                eq_(self.changed.name, 'override_name')
+
+            def root_namespace_object_name_overrides_module_name(self):
+                # Duplicates part of previous test for explicitness' sake.
+                # I.e. proves that the name doesn't end up 'explicit_root'.
+                eq_(self.unchanged.name, 'builtin_name')
+
+
     class add_task:
         def setup(self):
             self.c = Collection()
@@ -142,6 +209,17 @@ class Collection_(Spec):
             self.c.add_collection(Collection('sub'))
             self.c.add_task(_mytask, 'sub')
 
+        def allows_specifying_task_defaultness(self):
+            self.c.add_task(_mytask, default=True)
+            eq_(self.c.default, '_mytask')
+
+        def specifying_default_False_overrides_task_setting(self):
+            @task(default=True)
+            def its_me():
+                pass
+            self.c.add_task(its_me, default=False)
+            eq_(self.c.default, None)
+
     class add_collection:
         def setup(self):
             self.c = Collection()
@@ -282,24 +360,24 @@ class Collection_(Spec):
 
         def basic_set_and_get(self):
             self.c.configure({'foo': 'bar'})
-            eq_(self.c.configuration, {'foo': 'bar'})
+            eq_(self.c.configuration(), {'foo': 'bar'})
 
         def configure_performs_merging(self):
             self.c.configure({'foo': 'bar'})
-            eq_(self.c.configuration['foo'], 'bar')
+            eq_(self.c.configuration()['foo'], 'bar')
             self.c.configure({'biz': 'baz'})
-            eq_(set(self.c.configuration.keys()), set(['foo', 'biz']))
+            eq_(set(self.c.configuration().keys()), set(['foo', 'biz']))
 
         def configure_allows_overwriting(self):
             self.c.configure({'foo': 'one'})
-            eq_(self.c.configuration['foo'], 'one')
+            eq_(self.c.configuration()['foo'], 'one')
             self.c.configure({'foo': 'two'})
-            eq_(self.c.configuration['foo'], 'two')
+            eq_(self.c.configuration()['foo'], 'two')
 
-        def access_returns_dict(self):
-            eq_(self.c.configuration, {})
+        def call_returns_dict(self):
+            eq_(self.c.configuration(), {})
             self.c.configure({'foo': 'bar'})
-            eq_(self.c.configuration, {'foo': 'bar'})
+            eq_(self.c.configuration(), {'foo': 'bar'})
 
         def access_merges_from_subcollections(self):
             inner = Collection('inner')
@@ -307,24 +385,53 @@ class Collection_(Spec):
             outer = Collection()
             outer.configure({'biz': 'baz'})
             # With no inner collection
-            eq_(set(outer.configuration.keys()), set(['biz']))
+            eq_(set(outer.configuration().keys()), set(['biz']))
             # With inner collection
             outer.add_collection(inner)
-            eq_(set(outer.configuration.keys()), set(['foo', 'biz']))
+            eq_(set(outer.configuration('inner').keys()), set(['foo', 'biz']))
 
-        def parents_overwrite_children(self):
+        def parents_overwrite_children_in_path(self):
             inner = Collection('inner')
             inner.configure({'foo': 'inner'})
             outer = Collection()
             outer.add_collection(inner)
-            eq_(outer.configuration['foo'], 'inner')
+            eq_(outer.configuration('inner')['foo'], 'inner')
             outer.configure({'foo': 'outer'})
-            eq_(outer.configuration['foo'], 'outer')
+            eq_(outer.configuration('inner')['foo'], 'outer')
 
-        def access_merges_siblings_in_alpha_order(self):
-            a = Collection('a')
-            a.configure({'mysetting': 'a'})
-            b = Collection('b')
-            b.configure({'mysetting': 'b'})
-            parent = Collection(b, a) # Ensuring add order doesn't matter. Eh.
-            eq_(parent.configuration['mysetting'], 'b')
+        def sibling_subcollections_ignored(self):
+            inner = Collection('inner')
+            inner.configure({'foo': 'hi there'})
+            inner2 = Collection('inner2')
+            inner2.configure({'foo': 'nope'})
+            outer = Collection(inner, inner2)
+            eq_(outer.configuration('inner')['foo'], 'hi there')
+            eq_(outer.configuration('inner2')['foo'], 'nope')
+
+        def subcollection_paths_may_be_dotted(self):
+            leaf = Collection('leaf')
+            leaf.configure({'key': 'leaf-value'})
+            middle = Collection('middle', leaf)
+            outer = Collection('outer', middle)
+            eq_(outer.configuration('middle.leaf'), {'key': 'leaf-value'})
+
+        def invalid_subcollection_paths_result_in_KeyError(self):
+            # Straight up invalid
+            assert_raises(KeyError, Collection('meh').configuration, 'nope')
+            # Exists but wrong level
+            inner = Collection('inner')
+            inner.configure({'foo': 'bar'})
+            assert_raises(KeyError,
+                Collection('outer', inner).configuration, 'foo')
+
+        def keys_dont_have_to_exist_in_full_path(self):
+            # Kinda duplicates earlier stuff; meh
+            # Key only stored on leaf
+            leaf = Collection('leaf')
+            leaf.configure({'key': 'leaf-value'})
+            middle = Collection('middle', leaf)
+            outer = Collection('outer', middle)
+            eq_(outer.configuration('middle.leaf'), {'key': 'leaf-value'})
+            # Key stored on mid + leaf but not outer
+            middle.configure({'key': 'whoa'})
+            eq_(outer.configuration('middle.leaf'), {'key': 'whoa'})
